package com.elab.mq.listener;

import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.elab.core.exception.BusinessException;
import com.elab.core.utils.DateUtils;
import com.elab.core.utils.ObjectUtils;
import com.elab.mq.dao.IConsumerDao;
import com.elab.mq.dao.IProducerDao;
import com.elab.mq.model.ConsumerEntity;
import com.elab.mq.model.MessageModel;
import com.elab.mq.model.ProducerEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 抽象成客户端能够使用的方法
 *
 * @author ： liukx
 * @time ： 2019/4/22 - 18:02
 */
public abstract class AbstractMessageListener implements MessageListener {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private IProducerDao producerDao;

    @Autowired
    private IConsumerDao consumerDao;


    /**
     * 客户端关注的topic消息,实现了该方法,消息监听被触发时会与之匹配。
     *
     * @return
     */
    public abstract String topic();

    /**
     * 客户端关注的topic下的tag子类标签
     *
     * @return
     */
    public String tag() {
        return "*";
    }


    private ConsumerEntity init(String producerId) throws Exception {
        ConsumerEntity oldConsumerEntity = new ConsumerEntity();
        ProducerEntity producerEntity = producerDao.selectById(producerId);
        logger.info("获取生产者的信息：" + ObjectUtils.objectParseJsonStr(producerEntity));
        if (producerEntity != null) {
            logger.info("进入消费者验证...");
            ConsumerEntity consumerEntity = new ConsumerEntity();
            consumerEntity.setProducerId(producerEntity.getId());
            consumerEntity.setTopicId(producerEntity.getTopicId());
            consumerEntity.setStatus(1);
            oldConsumerEntity = consumerDao.selectByObject(consumerEntity);
            if (oldConsumerEntity != null && oldConsumerEntity.getConsumerStatus().equals(1)) {
                logger.info("消费者验证失败，已经消费成功过一次，不允许重复消费");
                throw new BusinessException("请不要重复消费，key：" + producerId);
            }
            logger.info("消费者验证通过，消费者消费的数据匹配上生产者数据，");
            if (oldConsumerEntity != null) {
                oldConsumerEntity.setRetryCount(oldConsumerEntity.getRetryCount() + 1);
                oldConsumerEntity.setConsumerStatus(-1);
                oldConsumerEntity.setUpdated(DateUtils.getCurrentDateTime());
                consumerDao.updateById(oldConsumerEntity);
            } else {
                oldConsumerEntity = new ConsumerEntity();
                oldConsumerEntity.setProducerId(producerEntity.getId());
                oldConsumerEntity.setContent(producerEntity.getContent());
                oldConsumerEntity.setHouseId(producerEntity.getHouseId());
                oldConsumerEntity.setModuleName(producerEntity.getModuleName());
                oldConsumerEntity.setStatus(1);
                oldConsumerEntity.setCreated(DateUtils.getCurrentDateTime());
                oldConsumerEntity.setRetryCount(0);
                oldConsumerEntity.setTopicId(producerEntity.getTopicId());
                oldConsumerEntity.setConsumerStatus(0);
                int id = consumerDao.insert(oldConsumerEntity);
                oldConsumerEntity.setId(id);
            }
        } else {
            logger.warn("生产者为空，无法匹配消费者");
        }
        return oldConsumerEntity;
//        ConsumerEntity consumerEntity = new ConsumerEntity();

    }


    /**
     * 客户端的业务逻辑实现
     *
     * @param message        消息对象
     * @param consumeContext 消息上下文
     * @return
     */
    public abstract Action consume0(MessageModel message, ConsumeContext consumeContext) throws Exception;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        MessageModel messageModel = new MessageModel(message);
        logger.debug("消息处理被触发 : " + message.toString());
        Action action = null;
        try {
            ConsumerEntity oldConsumerEntity = init(message.getKey());
            logger.info("更新消费者数据： " + ObjectUtils.objectParseJsonStr(oldConsumerEntity));
            action = consume0(messageModel, consumeContext);

            if (Action.ReconsumeLater.equals(action)) {
                oldConsumerEntity.setConsumerStatus(-1);
                consumerDao.updateById(oldConsumerEntity);
            } else {
                oldConsumerEntity.setConsumerStatus(1);
                consumerDao.updateById(oldConsumerEntity);
            }
        } catch (Exception e) {
            logger.debug("消息处理异常 : " + action);
            e.printStackTrace();
            return Action.ReconsumeLater;
        }
        logger.debug("消息处理结果 : " + action);
        return action;
    }

}
